#!/bin/bash
ELASTIC_SEARCH_LABEL='Elasticsearch'

#Print the input with additional formatting to indicate a section/title
title () {
    echo
    echo "-----------------------------------------------------"
    printf "| %-50s|\n" "$1"
    echo "-----------------------------------------------------"
}

# This function prints the echo with color.
#If invoked with a single string, treats it as INFO level
#Valid inputs for the second parameter are DEBUG and INFO
log() {
    echo ""
    echo -e $1
    echo ""
}

# Utility method to display warnings and important information
warn() {
    echo ""
    echo -e "\033[33m $1 \033[0m"
    echo ""
}

errorExit() {
    echo; echo -e "\033[31mERROR:$1\033[0m"; echo
    exit 1
}

attempt_number=0
elasticSearchIsNotReady(){
	#echo "in method isElasticSearchReady"
	curl "$ELASTIC_SEARCH_URL" > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		if [ $attempt_number -gt 10 ]; then
			errorExit "Unable to proceed. $ELASTIC_SEARCH_LABEL is not reachable. The command [curl $ELASTIC_SEARCH_URL] is failing. Gave up after $attempt_number attempts"
		fi
		let "attempt_number=attempt_number+1"
		return 0
	else
		return 1
	fi
}

runCommand() {
	curl_response=
	local operation=$1
	local commandToRun=$2
	local request_body=$3
	local params=$4
	
	commandToRun="\"$ELASTIC_SEARCH_URL/$commandToRun\"" 
	if [[ ! -z "$ELASTIC_SEARCH_USERNAME" && ! -z "$ELASTIC_SEARCH_PASSWORD" ]]; then
		commandToRun="$commandToRun --user $ELASTIC_SEARCH_USERNAME:$ELASTIC_SEARCH_PASSWORD"
	fi
	
	if [[ ! -z "$params" ]]; then
		commandToRun="$commandToRun $params"
	fi
	
	if [[ ! -z "$request_body" ]]; then
		commandToRun="$commandToRun -d '"$request_body"'"
	fi
	
	if [[ "$operation" == "GET" ]]; then
		commandToRun="curl --silent -XGET $commandToRun"
		curl_response=$(eval "${commandToRun}") 
	else
		eval "curl -X $operation ${commandToRun}" || errorExit "could not update Elastic Search"
	fi
}

setElasticSearchParams() {
	log "Waiting for $ELASTIC_SEARCH_LABEL to get ready (using the command: [curl $ELASTIC_SEARCH_URL])"
	while elasticSearchIsNotReady
	do
			sleep 5
			echo -n '.'
	done
	log "$ELASTIC_SEARCH_LABEL is ready. Executing commands"
	runCommand "GET" "_alias/active_insight_data"
	#echo "$ELASTIC_SEARCH_LABEL curl response: $curl_response"
	if [[ $curl_response = *missing* ]]; then
	    log "Running alias creation"
	    runCommand "PUT" "_template/active_insight_data" '{"template":"active_insight_data_*","aliases":{"active_insight_data":{},"search_insight_data":{}},"mappings":{"repo_storage_info":{"properties":{"used_space":{"type":"double"},"timestamp":{"type":"date"}}},"storage_summary_info":{"properties":{"used_space":{"type":"double"},"artifacts_size":{"type":"double"},"timestamp":{"type":"date"}}}}}' '-H "Content-Type: application/json"'
	    runCommand "PUT" "%3Cactive_insight_data_%7Bnow%2Fd%7D-1%3E"
	else
		runCommand "GET" "_template/active_insight_data"
		if [[ $curl_response != *double* ]]; then
			updatetemplate
		fi	
	fi
	log "$ELASTIC_SEARCH_LABEL setup is now complete"
}

updatetemplate(){
	log "Updating template" 
    runCommand "PUT" "_template/active_insight_data" '{"template": "active_insight_data_*","aliases": {"active_insight_data":  {},"search_insight_data": {}},"mappings":{"repo_storage_info":{"properties":{"used_space":{"type":"double"},"timestamp":{"type":"date"}}},"storage_summary_info":{"properties":{"used_space":{"type":"double"},"artifacts_size":{"type":"double"},"timestamp":{"type":"date"}}}}}' '-H "Content-Type: application/json"'
    log "Updated template" 

    runCommand "GET" "_alias/active_insight_data"
    indexname=$(echo $curl_response |cut -d'"' -f 2)
   	log "Old index $indexname"
    runCommand "PUT" "%3Cactive_insight_data_%7Bnow%2Fd%7D-1%3E"
    log "Created new index" 

    runCommand "GET" "_alias/active_insight_data"
    log $aliasdetails
    runCommand "POST" "_aliases" '{"actions":[{"remove":{"index":"'$indexname'","alias":"active_insight_data"}}]}' '-H "Content-Type: application/json"'
    log "Removed the old index from active alias"
}

main() {
	if [[ -z $ELASTIC_SEARCH_URL ]]; then
		title "$ELASTIC_SEARCH_LABEL Manual Setup"
		log "This script will attempt to seed $ELASTIC_SEARCH_LABEL with the templates and indices needed by JFrog Mission Control"

		warn "Please enter the same details as you entered during installation. If the details are incorrect, you may need to rerun the installation"

		local DEFAULT_URL="http://docker.for.mac.localhost:9200"
	    read -p  "Please enter the $ELASTIC_SEARCH_LABEL URL [$DEFAULT_URL]:" choice
	    : ${choice:=$DEFAULT_URL}
	    ELASTIC_SEARCH_URL=$choice
	fi
	echo "Beginning $ELASTIC_SEARCH_LABEL bootstrap"
	setElasticSearchParams
}

main

