#!/bin/bash
POSTGRES_LABEL='Postgres'


#Print the input with additional formatting to indicate a section/title
title () {
    echo
    echo "-----------------------------------------------------"
    printf "| %-50s|\n" "$1"
    echo "-----------------------------------------------------"
}

# This function prints the echo with color.
#If invoked with a single string, treats it as INFO level
#Valid inputs for the second parameter are DEBUG and INFO
log() {
    echo ""
    echo -e $1
    echo ""
}

# Utility method to display warnings and important information
warn() {
    echo ""
    echo -e "\033[33m $1 \033[0m"
    echo ""
}

#Display an error and exit
errorExit() {
	echo; echo  -e "\033[31mERROR: $1\033[0m"; echo
    exit 1
}

attempt_number=0
isStarting() {
	psql -U $POSTGRES_USER -l > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		if [ $attempt_number -gt 20 ]; then
            errorExit "Unable to proceed. $POSTGRES_LABEL is not reachable. This can occur if the service is not running. Gave up after $attempt_number attempts"
        fi
		let "attempt_number=attempt_number+1"
		return 0
	else
		return 1
	fi
}

INSTALLER_DIR="$( cd "$( dirname $0 )/" && pwd )"


createUsers() {
	log "Waiting for $POSTGRES_LABEL to get ready using the command: [psql -U $POSTGRES_USER -l]"
	while isStarting
	do
			sleep 5
			echo -n '.'
	done
	log "$POSTGRES_LABEL is now up. Checking if it already has data"
	psql -d $POSTGRES_DB -U $POSTGRES_USER -c 'select count(*) from qrtz_locks;' -A -t > /dev/null 2>&1
	outcome=$?
	if [ $outcome -eq 1 ]; then
		log "Beginning $POSTGRES_LABEL data seeding"
		psql -d $POSTGRES_DB -U $POSTGRES_USER -a -f $INSTALLER_DIR/quartz_postgres.sql > /dev/null
		EXIT_CODE=$?
		if [[ $EXIT_CODE -eq 0 ]]; then
			log "$POSTGRES_LABEL setup is now complete"
		fi
		exit $EXIT_CODE
	elif [ $outcome -ne 0 ]; then
		errorExit "Could not create $POSTGRES_LABEL tables. Please ensure that the user quartzdb exists and has full permissions on the database quartzdb"
	else
		log "Bootstrap tables already exist"
	fi
}

# Utility method to get user's input
# 1st parameter "what to ask the user"
# 3rd parameter "what is the default choice (to use if the user simply presses Enter)"
# Example 'getUserInput "Are you feeling lucky? Punk!" "y"'
getUserInput(){
    
    local choice="na"
    local text_to_display=$1
    local default_choice=$2
    users_choice=
    
    if [ ! -z $default_choice ]; then
        text_to_display="$1 [$default_choice]:"
    else
        text_to_display="$1 :"
    fi

    echo "";echo "";
    sleep 1
    read -p "$text_to_display" choice
    if [ ! -z $default_choice ]; then
        : ${choice:=$default_choice}
    fi
    users_choice=$choice
    sleep 5 #This ensures correct logging
}

getPostgresDetails() {

    getUserInput "Please enter the path where $POSTGRES_LABEL executable (psql) is available" "$POSTGRES_PATH"
    POSTGRES_PATH=$users_choice
    
    getUserInput "Please enter the ID of the root user" "$POSTGRES_ROOT_USER_ID"
    POSTGRES_ROOT_USER_ID=$users_choice

    getUserInput "Please enter the password of the root user" "$POSTGRES_ROOT_USER_PWD"
    POSTGRES_ROOT_USER_PWD=$users_choice
}

main() {
    if [[ -z $POSTGRES_DB || $POSTGRES_DB != "quartzdb" ]]; then

		if [[ -z $POSTGRES_ROOT_USER_ID ]]; then
	    	title "$POSTGRES_LABEL Setup"
			log "This script will attempt to seed $POSTGRES_LABEL with the tables needed by JFrog Mission Control."
			getPostgresDetails
		fi

		export POSTGRES_PATH="$POSTGRES_PATH"
    	export PATH="$POSTGRES_PATH:$PATH"

        if [[ ! -z "$POSTGRES_OMNI_DB" && "$POSTGRES_OMNI_DB" != "" ]]; then
    		export POSTGRES_DB="$POSTGRES_OMNI_DB"
        else
            export POSTGRES_DB="quartzdb"
        fi

            
		export POSTGRES_DB="quartzdb"
        export POSTGRES_USER="quartzdb"
        export POSTGRES_PWD="password"
        export POSTGRES_ROOT_USER_ID="$POSTGRES_ROOT_USER_ID"
        export POSTGRES_ROOT_USER_PWD="$POSTGRES_ROOT_USER_PWD"
       
        # Create quartzdb user on PostgreSQL
        log "Creating $POSTGRES_USER user on PostgreSQL"

        chown -R $POSTGRES_ROOT_USER_ID:$POSTGRES_ROOT_USER_ID ${INSTALLER_DIR} || errorExit "Seeding failed due to errors"

        TMP_FILE=$(mktemp /tmp/tmp.XXXXXX.sh)
        cat << EOF > ${TMP_FILE}
#!/bin/bash
export PGPASSWORD="$POSTGRES_ROOT_USER_PWD"
export POSTGRES_USER="$POSTGRES_USER"
export POSTGRES_DB="$POSTGRES_DB"
export POSTGRES_PWD="$POSTGRES_PWD"

#printenv #(useful for debugging)
errorExit() {
    echo; echo -e "\033[31mERROR:\033[0m $1"; echo
    exit 1
}
$POSTGRES_PATH/psql $POSTGRES_ROOT_USER_ID -tAc 'SELECT 1 FROM pg_roles WHERE rolname='\''$POSTGRES_USER'\''' | grep -q 1 || $POSTGRES_PATH/psql -c 'CREATE USER '$POSTGRES_USER' WITH PASSWORD '\''$POSTGRES_PWD'\'';' || errorExit "Failed creating user $POSTGRES_USER on PostgreSQL"
if $POSTGRES_PATH/psql -lqt | cut -d \| -f 1 | grep -qw $POSTGRES_USER; then
    echo "Nothing to do"
else
    $POSTGRES_PATH/psql -c "CREATE DATABASE $POSTGRES_DB WITH OWNER=$POSTGRES_USER ENCODING='UTF8' TEMPLATE template0;" || errorExit "Failed creating db $POSTGRES_DB on PostgreSQL"
fi
$POSTGRES_PATH/psql -c "GRANT ALL PRIVILEGES ON DATABASE $POSTGRES_DB TO $POSTGRES_USER;" || errorExit "Failed granting all privileges on $POSTGRES_DB to user $POSTGRES_USER on PostgreSQL"
exit 0
EOF
#END of included content
        chmod +xr ${TMP_FILE} || errorExit "Adding permissions to ${TMP_FILE} failed"
        chown $POSTGRES_ROOT_USER_ID:$POSTGRES_ROOT_USER_ID ${TMP_FILE}   || errorExit "Seeding failed due to errors"
        sudo su - $POSTGRES_ROOT_USER_ID -c "bash $TMP_FILE" || errorExit "Seeding failed due to errors"

        export PGPASSWORD="$POSTGRES_PWD"
    fi
    echo "Beginning $POSTGRES_LABEL bootstrap"
	createUsers
}
main